package com.vendor.platform.ui.internal;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

public final class ReflectUtil {
  public static boolean isStatic(Method method) {
    return ((method.getModifiers() & 0x8) > 0);
  }
  
  public static boolean isAbstract(Class classToCheck) {
    return ((classToCheck.getModifiers() & 0x400) > 0);
  }
  
  public static Method getMethod(Class parentClass, String methodName) {
    return getMethod(parentClass, methodName, new Class[0]);
  }
  
  public static Method getMethod(Class parentClass, String methodName, Class parameterType) {
    return getMethod(parentClass, methodName, new Class[] { parameterType });
  }
  
  public static Method getMethod(Class parentClass, String methodName, Class... parameterTypes) {
    try {
      return parentClass.getMethod(methodName, parameterTypes);
    } catch (NoSuchMethodException e) {
      throw new RuntimeException(e);
    } 
  }
  
  public static Method[] getMethodsWithName(Class parentClass, String methodName) {
    List<Method> list = new ArrayList<>();
    Method[] methods = parentClass.getMethods();
    for (int i = 0; i < methods.length; i++) {
      Method method = methods[i];
      if (method.getName().equals(methodName))
        list.add(method); 
    } 
    return list.<Method>toArray(new Method[list.size()]);
  }
  
  public static Constructor getConstructor(Class parentClass, Class... parameterTypes) {
    try {
      return parentClass.getConstructor(parameterTypes);
    } catch (NoSuchMethodException e) {
      throw new RuntimeException(e);
    } 
  }
  
  public static Object getInstance(String name, String method, Object... params) {
    try {
      Class<?> c = Class.forName(name);
      Method m = getMethod(c, method, params);
      if (m == null)
        return null; 
      return m.invoke(null, params);
    } catch (RuntimeException e) {
      throw e;
    } catch (Exception e) {
      throw new RuntimeException(e);
    } 
  }
  
  public static Object newInstance(String name, Object... params) {
    try {
      return newInstance(Class.forName(name), params);
    } catch (ClassNotFoundException e) {
      throw new RuntimeException(e);
    } 
  }
  
  public static Object newInstance(Class parentClass, Object... params) {
    try {
      Class[] parameterTypes = new Class[params.length];
      for (int i = 0; i < params.length; i++)
        parameterTypes[i] = params[i].getClass(); 
      Constructor c = getConstructor(parentClass, parameterTypes);
      return c.newInstance(params);
    } catch (Exception e) {
      throw new RuntimeException(e);
    } 
  }
  
  public static Object invoke(Object obj, String method) {
    try {
      Method m = getMethod(obj.getClass(), method);
      return m.invoke(obj, new Object[0]);
    } catch (RuntimeException e) {
      throw e;
    } catch (Exception e) {
      throw new RuntimeException(e);
    } 
  }
  
  public static Object invoke(Object obj, String method, Object... params) {
    try {
      Method m = getMethod(obj.getClass(), method, params);
      if (m == null)
        return null; 
      return m.invoke(obj, params);
    } catch (RuntimeException e) {
      throw e;
    } catch (Exception e) {
      throw new RuntimeException(e);
    } 
  }
  
  public static Method getMethod(Class cls, String method, Object... params) {
    Method[] methods = getMethodsWithName(cls, method);
    if (methods == null || methods.length == 0)
      return null; 
    if (methods.length == 1)
      return methods[0]; 
    for (Method m : methods) {
      Class<?>[] types = m.getParameterTypes();
      if (isMatch(types, params))
        return m; 
    } 
    return null;
  }
  
  private static boolean isMatch(Class<?>[] types, Object[] params) {
    if (types.length != params.length)
      return false; 
    for (int i = 0; i < types.length; i++) {
      if (!isMatch(types[i], params[i]))
        return false; 
    } 
    return true;
  }
  
  private static boolean isMatch(Class<?> type, Object obj) {
    if (type.isPrimitive()) {
      if (type == int.class && obj.getClass() == Integer.class)
        return true; 
      if (type == long.class && obj.getClass() == Long.class)
        return true; 
      if (type == boolean.class && obj.getClass() == Boolean.class)
        return true; 
      if (type == float.class && obj.getClass() == Float.class)
        return true; 
      if (type == double.class && obj.getClass() == Double.class)
        return true; 
      if (type == byte.class && obj.getClass() == Byte.class)
        return true; 
      if (type == short.class && obj.getClass() == Short.class)
        return true; 
      if (type == char.class && obj.getClass() == Character.class)
        return true; 
    } 
    return type.isAssignableFrom(obj.getClass());
  }
}
